@@ -1,5 +1,4 @@
 from robomotion.runtime import Runtime
-import pyperclip
 
 class NodeFactory:
     def __init__(self, c):
@@ -7,8 +6,5 @@ class NodeFactory:
 
     def on_create(self, config: bytes):
         node = Runtime.deserialize(config, self.c)
-        
-        pyperclip.copy(config.decode())
-        
         Runtime.add_node(node.guid, node)
         node.on_create()
