@@ -18,7 +18,7 @@ from robomotion.spec import Spec
 from robomotion.debug import Debug
 
 
-async def start():
+def start():
     attached = False
     config = {}
     ns = ''
