@@ -1,9 +1,12 @@
-import inspect, json, sys
+import inspect
+import json
+import sys
 from robomotion.node import Node
 from robomotion.variable import Variable, InVariable, OptVariable, OutVariable, Credentials, ECategory, _Enum
 
+
 class NodeSpec:
-    def __init__(self, id: str, icon: str='', name: str='', color: str='', editor: str=None, inputs: int=0, outputs: int=0):
+    def __init__(self, id: str, icon: str = '', name: str = '', color: str = '', editor: str = None, inputs: int = 0, outputs: int = 0):
         self.id = id
         self.icon = icon
         self.name = name
@@ -33,6 +36,7 @@ class Schema:
 class SProperty:
     type = ''
     title = ''
+    description = None
     subtitle = None
     category = None
     properties = None
@@ -58,7 +62,8 @@ class Spec:
     def generate(plugin_name, version):
         frm = inspect.stack()[2]
         mod = inspect.getmodule(frm[0])
-        clsmembers = inspect.getmembers(sys.modules[mod.__name__], inspect.isclass)
+        clsmembers = inspect.getmembers(
+            sys.modules[mod.__name__], inspect.isclass)
 
         pspec = {'name': plugin_name, 'version': version}
 
@@ -115,11 +120,19 @@ class Spec:
                             inObject['messageOnly'] = True
                         if input.pyScope:
                             inObject['pyScope'] = True
+                        if input.description != '':
+                            inObject['description'] = input.description
 
                         name = Spec.lower_first_letter(_input['key'])
+                        if input.format != '':
+                            inObject['format'] = input.format
+                            pUISchema[name] = {'ui:field': input.format}
+                        else:
+                            pUISchema[name] = {'ui:field': 'variable'}
+
                         inProperties[name] = inObject
-                        pUISchema[name] = {'ui:field': 'variable'}
-                        formData[name] = {'scope': input.scope, 'name': input.name}
+                        formData[name] = {
+                            'scope': input.scope, 'name': input.name}
                         uiOrder.append(name)
 
                     for _input in inputs:
@@ -132,6 +145,12 @@ class Spec:
                         name = Spec.lower_first_letter(_input['key'])
                         if input.hidden:
                             pUISchema[name] = {'ui:widget': 'hidden'}
+                        if input.description != '':
+                            inObject['description'] = input.description
+                            pUISchema[name] = {'ui:field': 'input'}
+                        if input.format != '':
+                            inObject['format'] = input.format
+                            pUISchema[name] = {'ui:field': input.format}
 
                         inProperties[name] = inObject
                         formData[name] = input.default
@@ -179,11 +198,14 @@ class Spec:
                             outObject['messageScope'] = True
                         if output.messageOnly:
                             outObject['messageOnly'] = True
+                        if output.description != '':
+                            outObject['description'] = output.description
 
                         name = Spec.lower_first_letter(_output['key'])
                         outProperties[name] = outObject
                         pUISchema[name] = {'ui:field': 'variable'}
-                        formData[name] = {'scope': output.scope, 'name': output.name}
+                        formData[name] = {
+                            'scope': output.scope, 'name': output.name}
                         uiOrder.append(name)
 
                     for _output in outputs:
@@ -196,6 +218,9 @@ class Spec:
                         name = Spec.lower_first_letter(_output['key'])
                         if output.hidden:
                             pUISchema[name] = {'ui:widget': 'hidden'}
+                        if output.description != '':
+                            outObject['description'] = output.description
+                            pUISchema[name] = {'ui:field': 'input'}
 
                         outProperties[name] = outObject
                         formData[name] = output.default
@@ -248,11 +273,19 @@ class Spec:
                             optObject['messageOnly'] = True
                         if option.pyScope:
                             optObject['pyScope'] = True
+                        if option.description != '':
+                            optObject['description'] = option.description
 
                         name = Spec.lower_first_letter(_option['key'])
+                        if option.format != '':
+                            optObject['format'] = option.format
+                            pUISchema[name] = {'ui:field': option.format}
+                        else:
+                            pUISchema[name] = {'ui:field': 'variable'}
+
                         optProperties[name] = optObject
-                        pUISchema[name] = {'ui:field': 'variable'}
-                        formData[name] = {'scope': option.scope, 'name': option.name}
+                        formData[name] = {
+                            'scope': option.scope, 'name': option.name}
                         uiOrder.append(name)
 
                     for _option in options:
@@ -265,6 +298,12 @@ class Spec:
                         category = option.category
                         if category != ECategory.Null:
                             optObject['category'] = int(category)
+                        if option.description != '':
+                            optObject['description'] = option.description
+                            pUISchema[name] = {'ui:field': 'input'}
+                        if option.format != '':
+                            optObject['format'] = option.format
+                            pUISchema[name] = {'ui:field': option.format}
 
                         enums = option.enum.enums
                         if enums is not None and len(enums) > 0:
@@ -290,21 +329,37 @@ class Spec:
                             'title': credential.title,
                             'subtitle': credential.title,
                             'category': int(credential.category),
+                            'customScope': True,
+                            'messageScope': True,
                             'properties': {
-                                'vaultId': {
-                                    'type': 'string'
+                                'scope': {
+                                    'type': 'string',
+                                },
+                                'name': {
+                                    'type': 'object',
+                                    'properties': {
+                                        'vaultId': {
+                                            'type': 'string'
+                                        },
+                                        'itemId': {
+                                            'type': 'string'
+                                        }
+                                    }
                                 },
-                                'itemId': {
-                                    'type': 'string'
-                                }
                             }
                         }
 
+                        if credential.description != '':
+                            optObject['description'] = credential.description
+
                         name = Spec.lower_first_letter(_credential['key'])
-                        pUISchema[name] = {'ui:field': 'credentials'}
+                        pUISchema[name] = {'ui:field': 'vault'}
                         formData[name] = {
-                            'vaultId': '_',
-                            'itemId': '_'
+                            'scope': 'Custom',
+                            'name': {
+                                'vaultId': '_',
+                                'itemId': '_'
+                            }
                         }
                         optProperties[name] = optObject
                         uiOrder.append(name)
